@@ -483,7 +483,7 @@ SEXP rnng_get_opt(SEXP object, SEXP opt) {
     }
 
   } else {
-    Rf_error("'object' is not a valid Socket, Context, Stream, Listener or Dialer");
+    Rf_error("'object' is not a valid Socket, Context, Stream, Listener, or Dialer");
   }
 
   if (xc)
@@ -589,6 +589,8 @@ SEXP rnng_serial_config(SEXP klass, SEXP sfunc, SEXP ufunc, SEXP vec) {
 
 }
 
+// specials --------------------------------------------------------------------
+
 SEXP rnng_set_marker(SEXP x) {
 
   special_bit = (uint8_t) NANO_INTEGER(x);
@@ -596,8 +598,6 @@ SEXP rnng_set_marker(SEXP x) {
 
 }
 
-// specials --------------------------------------------------------------------
-
 SEXP rnng_advance_rng_state(void) {
 
   GetRNGstate();
@@ -607,9 +607,27 @@ SEXP rnng_advance_rng_state(void) {
 
 }
 
+SEXP rnng_interrupt_switch(SEXP x) {
+
+  nano_interrupt = NANO_INTEGER(x);
+  return x;
+
+}
+
 SEXP rnng_fini(void) {
 
   nng_fini();
   return R_NilValue;
 
 }
+
+SEXP rnng_traverse_precious(void) {
+
+  SEXP list = nano_precious;
+  int x = 0;
+  for (SEXP head = CDR(list); head != R_NilValue; head = CDR(head))
+    if (TAG(head) == R_NilValue) ++x;
+
+  return Rf_ScalarInteger(x);
+
+}
